@@ -10,9 +10,7 @@
  *
  * USAGE:
  *
- * newrole -r <role> [-- optional args to shell...]
- * newrole -t <type> [-- optional args to shell...]
- * newrole -r <role> -t <type> [-- optional args to shell...]
+ * newrole [ -r role ] [ -t type ] [ -l level ] [ -- args ]
  *
  * BUILD OPTIONS:
  *
@@ -64,15 +62,19 @@
 #include <selinux/get_default_type.h>
 #include <selinux/get_context_list.h> /* for SELINUX_DEFAULTUSER */
 #include <signal.h>
+#ifdef USE_NLS
 #include <locale.h>			    /* for setlocale() */
 #include <libintl.h>			    /* for gettext() */
 #define _(msgid) gettext (msgid)
+#else
+#define _(msgid) (msgid)
+#endif
 #ifndef PACKAGE
 #define PACKAGE "policycoreutils"   /* the name of this package lang translation */
 #endif
 
 /* USAGE_STRING describes the command-line args of this program. */
-#define USAGE_STRING "USAGE: newrole -r role [ -t type ] [ -- args ]"
+#define USAGE_STRING "USAGE: newrole [ -r role ] [ -t type ] [ -l level ] [ -- args ]"
 
 #define DEFAULT_CONTEXT_SIZE 255  /* first guess at context size */
 
@@ -88,6 +90,48 @@ char *xstrdup(const char *s)
   return s2;
 }
 
+static char *
+build_new_range(char *newlevel, const char *range)
+{
+  char *newrangep = NULL, *tmpptr;
+  size_t len;
+
+  /* a missing or empty string */
+  if (!range || !strlen(range) || !newlevel || !strlen(newlevel))
+    return NULL;
+
+  /* if the newlevel is actually a range - just use that */
+  if (strchr(newlevel, '-')) {
+      newrangep = strdup(newlevel);
+      return newrangep;
+  }
+
+  /* look for MLS range */
+  tmpptr = strchr(range, '-');
+
+  if (tmpptr) {
+    /* we are inserting into a ranged MLS context */
+    len = strlen(newlevel) + 1 + strlen(tmpptr + 1) + 1;
+    newrangep = (char *)malloc(len);
+    if (!newrangep)
+      return NULL;
+    snprintf(newrangep, len, "%s-%s", newlevel, tmpptr + 1);
+  } else {
+    /* we are inserting into a currently non-ranged MLS context */
+    if (!strcmp(newlevel, range)) {
+      newrangep = strdup(range);
+    } else {
+      len = strlen(newlevel) + 1 + strlen(range) + 1;
+      newrangep = (char *)malloc(len);
+      if (!newrangep)
+        return NULL;
+      snprintf(newrangep, len, "%s-%s", newlevel, range);
+    }
+  }
+
+  return newrangep;
+}
+
 #ifdef USE_PAM
 
 /************************************************************************
@@ -254,10 +298,12 @@ int main( int argc, char *argv[] ) {
   struct option long_options[] = {   /* long option flags for getopt() */
     { "role", 1, 0, 'r' },
     { "type", 1, 0, 't' },
+    { "level", 1, 0, 'l' },
     { NULL, 0, 0, 0 }
   };
   char *role_s = NULL;               /* role spec'd by user in argv[] */
   char *type_s = NULL;               /* type spec'd by user in argv[] */
+  char *level_s = NULL;              /* level spec'd by user in argv[] */
   char *ttyn   = NULL;		     /* tty path */
   pid_t childPid=0;			     
   int fd=0;
@@ -265,9 +311,11 @@ int main( int argc, char *argv[] ) {
   /* Terminate on SIGHUP. */
   signal(SIGHUP, SIG_DFL);
 
+#ifdef USE_NLS
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
+#endif
 
   /*
    *
@@ -283,7 +331,7 @@ int main( int argc, char *argv[] ) {
   }
 
   while (1) {
-    clflag=getopt_long(argc,argv,"r:t:",long_options,&flag_index);
+    clflag=getopt_long(argc,argv,"r:t:l:",long_options,&flag_index);
     if (clflag == -1)
 	break;
     
@@ -305,7 +353,20 @@ int main( int argc, char *argv[] ) {
       }
       type_s = optarg;  /* save the type string spec'd by user */
       break;
-      
+
+    case 'l':
+      if(!is_selinux_mls_enabled() ) {
+        fprintf(stderr, _("Sorry, -l may be used with SELinux MLS support.\n"));
+        exit(-1);
+      }
+      /* If level_s is already set, the user spec'd multiple levels - bad. */
+      if(level_s) {
+	fprintf( stderr, _("Error: multiple levels specified\n"));
+	exit( -1 );
+      }
+      level_s = optarg;  /* save the level string spec'd by user */
+      break;
+
     default:
       fprintf(stderr, "%s\n",USAGE_STRING);
       exit(-1);
@@ -315,7 +376,7 @@ int main( int argc, char *argv[] ) {
   
   /* Verify that the combination of command-line arguments we were *
    * given is a viable one.                                        */
-  if( !(role_s || type_s) ) {
+  if( !(role_s || type_s || level_s) ) {
     fprintf(stderr, "%s\n",USAGE_STRING);
     exit(-1);
   }
@@ -439,6 +500,25 @@ int main( int argc, char *argv[] ) {
 #endif
   } /* if user specified new type */
 
+  /* If the user specified a new level on the command line (if `level_s'   *
+   * is set), then replace the old level in `context' with this new level. */
+  if(level_s) {
+    char *range_s = build_new_range(level_s, context_range_get(context));
+    if (!range_s) {
+      fprintf(stderr, _("failed to build new range with level %s\n"), level_s);
+      exit(-1);
+    }
+    if(context_range_set(context, range_s)) {
+      fprintf(stderr, _("failed to set new range %s\n"), range_s);
+      free(range_s);
+      exit(-1);
+    }
+    free(range_s);
+#ifdef CANTSPELLGDB
+    printf("Your new range is %s\n", context_range_get(context));
+#endif
+  } /* if user specified new level */
+
   /* The second step in creating the new context is to convert our modified *
    * `context' structure back to a context string and then to a Context.    */
 
