@@ -10,9 +10,11 @@
  * USAGE:
  * restorecon [-Rnv] pathname...
  * 
+ * -e   Specify directory to exclude
  * -n	Do not change any file labels.
  * -v	Show changes in file labels.  
  * -o filename save list of files with incorrect context
+ * -F	Force reset of context to match file_context for customizable files
  *
  * pathname...	The file(s) to label 
  *
@@ -41,34 +43,105 @@ static int verbose=0;
 static FILE *outfile=NULL;
 static char *progname;
 static int errors=0;
-static int recurse;
+static int recurse=0;
+static int force=0;
+
+#define MAX_EXCLUDES 100
+static int excludeCtr=0;
+struct edir {
+	char *directory;
+        size_t size;
+};
+static struct edir excludeArray[MAX_EXCLUDES];
+static int add_exclude(const char *directory) {
+  struct stat sb;
+  if(directory == NULL || directory[0] != '/') {
+    fprintf(stderr, "Full path required for exclude: %s.\n", 
+	    directory);
+    return 1;
+  }
+  if(lstat(directory, &sb)) {
+    fprintf(stderr, "Directory \"%s\" not found, ignoring.\n", directory);
+    return 0;
+  }
+  if ((sb.st_mode & S_IFDIR) == 0 ) {
+    fprintf(stderr, "\"%s\" is not a Directory: mode %o, ignoring\n", directory,sb.st_mode);
+    return 0;
+  }
+
+  if (excludeCtr == MAX_EXCLUDES) {
+    fprintf(stderr, "Maximum excludes %d exceeded.\n", MAX_EXCLUDES);
+    return 1;
+  }
+
+   excludeArray[excludeCtr].directory = strdup(directory);
+  if (!excludeArray[excludeCtr].directory) {
+    fprintf(stderr, "Out of memory.\n");
+    return 1;
+  }
+  excludeArray[excludeCtr++].size = strlen(directory);
+
+  return 0;
+}
+static int exclude(const char *file) {
+	int i=0;
+	for(i=0; i < excludeCtr; i++) { 
+		if (strncmp(file,excludeArray[i].directory,excludeArray[i].size)==0) {
+			if (file[excludeArray[i].size]==0 || 
+			    file[excludeArray[i].size]=='/') {
+				return 1;
+			}
+		}
+	}
+	return 0;
+}
+
+/* Compare two contexts to see if their differences are "significant",
+ * or whether the only difference is in the user. */
+static int only_changed_user(const char *a, const char *b)
+{
+	char *rest_a, *rest_b; /* Rest of the context after the user */
+	if (force) return 0;
+	if (!a || !b) return 0;
+	rest_a = strchr(a, ':');
+	rest_b = strchr(b, ':');
+	if (!rest_a || !rest_b) return 0;
+	return  (strcmp(rest_a, rest_b) == 0);
+}
 
 void usage(const char * const name)
 {	
   fprintf(stderr,
-	  "usage:  %s [-Rnv] [-f filename | pathname... ]\n",  name);
+	  "usage:  %s [-Rnv] [-e excludedir ] [-o filename ] [-f filename | pathname... ]\n",  name);
   exit(1);
 }
 int restore(char *filename) {
   int retcontext=0;
   int retval=0;
-  security_context_t scontext;
-  security_context_t prev_context;
+  security_context_t scontext=NULL;
+  security_context_t prev_context=NULL;
   int len=strlen(filename);
   struct stat st;
   char path[PATH_MAX+1];
+  int user_only_changed=0;
   /* 
      Eliminate trailing /
   */
   if (len > 0 && filename[len-1]=='/' && (strcmp(filename,"/") != 0)) {
     filename[len-1]=0;
   }
+
+  if (excludeCtr > 0 && exclude(filename)) {
+      return 0;
+  }
+
   if (lstat(filename, &st)!=0) {
     fprintf(stderr,"lstat(%s) failed: %s\n", filename,strerror(errno));
     return 1;
   }
   if (S_ISLNK(st.st_mode)) {
-    fprintf(stderr,"Warning! %s refers to a symbolic link, not following last component.\n", filename);
+    if (verbose>1)
+      fprintf(stderr,"Warning! %s refers to a symbolic link, not following last component.\n", filename);
     char *p = NULL, *file_sep;
     char *tmp_path = strdup(filename);
     if (!tmp_path) {
@@ -99,8 +172,13 @@ int restore(char *filename) {
     }
     filename = p;
   }
+  if (excludeCtr > 0 && exclude(filename)) {
+      return 0;
+  }
   retval = matchpathcon(filename, st.st_mode, &scontext);
   if (retval < 0) {
+    if (errno == ENOENT)
+      return 0;
     fprintf(stderr,"matchpathcon(%s) failed %s\n", filename,strerror(errno));
     return 1;
   } 
@@ -111,11 +189,16 @@ int restore(char *filename) {
   retcontext=lgetfilecon(filename,&prev_context);
   
   if (retcontext >= 0 || errno == ENODATA) {
-    if (retcontext < 0 || strcmp(prev_context,scontext) != 0) {
+    int customizable=0;
+    if (retcontext < 0) prev_context=NULL;
+    if (retcontext < 0 || force || 
+	(strcmp(prev_context,scontext) != 0 && 	
+	 !(customizable=is_context_customizable(prev_context) > 0))) {
       if (outfile) {
 	fprintf(outfile, "%s\n", filename);
       }
-      if (change) {
+      user_only_changed = only_changed_user(scontext, prev_context);
+      if (change && !user_only_changed) {
 	retval=lsetfilecon(filename,scontext);
       }
       if (retval<0) {
@@ -126,10 +209,16 @@ int restore(char *filename) {
 	  freecon(scontext);
 	  return 1;
       } else 	
-	if (verbose)
-	  fprintf(stderr,"%s reset context %s->%s\n",
-		  progname, filename, scontext);
-    } 
+	      if (verbose && 
+		  (verbose > 1 || !user_only_changed))
+		      fprintf(stderr,"%s reset context %s:%s->%s\n",
+			      progname, filename, (retcontext >= 0 ? prev_context : ""), scontext);
+    }
+    if (verbose > 1 && customizable>0) {
+	    fprintf(stderr,"%s: %s not reset customized by admin to %s\n",
+		      progname, filename, prev_context);
+    }
+
     if (retcontext >= 0)
       freecon(prev_context);
   } 
@@ -142,7 +231,9 @@ int restore(char *filename) {
   return errors;
 }
 static int apply_spec(const char *file,
-		      const struct stat *sb_unused, int flag, struct FTW *s_unused)
+		      const struct stat *sb_unused __attribute__((unused)),
+		      int flag,
+		      struct FTW *s_unused __attribute__((unused)))
 {
 	if (flag == FTW_DNR) {
 		fprintf(stderr, "%s:  unable to read directory %s\n",
@@ -155,11 +246,11 @@ static int apply_spec(const char *file,
 void process(char *buf) {
       if (recurse) {
 	if (nftw
-	    (buf, apply_spec, 1024, FTW_PHYS | FTW_MOUNT)) {
+	    (buf, apply_spec, 1024, FTW_PHYS)) {
 	  fprintf(stderr,
 		  "%s:  error while labeling files under %s\n",
 		  progname, buf);
-	  exit(1);
+	  errors++;
 	}
       }
       else
@@ -173,13 +264,15 @@ int main(int argc, char **argv) {
   int opt;
   char buf[PATH_MAX];
 
+  memset(excludeArray,0, sizeof(excludeArray));
+
   progname=argv[0];
   if (is_selinux_enabled() <= 0 )
     exit(0);
 
   memset(buf,0, sizeof(buf));
 
-  while ((opt = getopt(argc, argv, "Rnvf:o:")) > 0) {
+  while ((opt = getopt(argc, argv, "FRnvf:o:e:")) > 0) {
     switch (opt) {
     case 'n':
       change = 0;
@@ -187,6 +280,12 @@ int main(int argc, char **argv) {
     case 'R':
       recurse = 1;
       break;
+    case 'F':
+      force = 1;
+      break;
+    case 'e':
+      if ( add_exclude(optarg) ) exit(1);
+      break;
     case 'o':
       outfile = fopen(optarg,"w");
       if (!outfile) {
@@ -196,7 +295,7 @@ int main(int argc, char **argv) {
       }
       break;
     case 'v':
-      verbose = 1;
+      verbose++;
       break;
     case 'f':
       file = 1;
