@@ -49,10 +49,14 @@
 #include <selinux/context.h>              /* for context-mangling functions */
 #include <fcntl.h>
 #include <ctype.h>
+#include <limits.h>
+#ifdef USE_NLS
 #include <libintl.h>
 #include <locale.h>
-#include <limits.h>
 #define _(msgid) gettext (msgid)
+#else
+#define _(msgid) (msgid)
+#endif
 #ifndef PACKAGE
 #define PACKAGE "policycoreutils"   /* the name of this package lang translation */
 #endif
@@ -314,9 +318,12 @@ int main( int argc, char *argv[] ) {
   extern char *optarg;               /* used by getopt() for arg strings */
   extern int opterr;                 /* controls getopt() error messages */
   security_context_t new_context;	     /* context for the init script context  */
+  
+#ifdef USE_NLS
   setlocale(LC_ALL, "");
   bindtextdomain(PACKAGE, LOCALEDIR);
   textdomain(PACKAGE);
+#endif
 
   /* Verify that we are running on a flask-enabled kernel. */
   if( !is_selinux_enabled() ) {
@@ -369,7 +376,15 @@ int main( int argc, char *argv[] ) {
        fprintf(stderr, _("Could not set exec context to %s.\n"), new_context);
        exit(-1);
   }
-  if ( execvp(argv[1], argv + 1) ) {
+  /*
+   * Do not execvp the command directly from run_init; since it would run
+   * under with a pty under sysadm_devpts_t. Instead, we call open_init_tty,
+   * which transitions us into initrc_t, which then spawns a new
+   * process, that gets a pty with context initrc_devpts_t. Just
+   * execvp or using a exec(1) recycles pty's, and does not open a new
+   * one. 
+   */
+  if ( execvp("/usr/sbin/open_init_pty", argv) ) {
     perror("execvp");
     exit(-1);
   }
