@@ -30,7 +30,7 @@
 #define BUFSIZE 255
 
 /* column to put the output (must be a multiple of 8) */
-#define COL 32
+static unsigned int COL=32;
 
 extern char *selinux_mnt;
 
@@ -152,15 +152,10 @@ out:
 }
 
 void printf_tab(const char *outp) {
-	/* print then add appropriate number of tabs */
-	int i, count;
+  char buf[20];
+  snprintf(buf, sizeof(buf), "%%-%us", COL);
+	printf(buf, outp);
 
-	printf("%s",outp);
-
-	/* insert the right number of tabs */
-	count=((COL - strlen(outp) -1) / 8);
-	for( i=0 ; i<count ; i++ )
-		printf("\t");
 }
 
 int main(int argc, char **argv) {
@@ -275,6 +270,10 @@ int main(int argc, char **argv) {
 	if(security_get_boolean_names(&bools,&nbool) >= 0) {
 		printf("\nPolicy booleans:\n");
 
+		for( i=0 ; i<nbool ; i++ ) {
+			if (strlen(bools[i]) + 1 > COL) 
+			    COL=strlen(bools[i])+1;
+		}
 		for( i=0 ; i<nbool ; i++ ) {
 			printf_tab(bools[i]);
 
